"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.makeTestEnvironment = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _lodash = _interopRequireDefault(require("lodash.partial"));

var _core = require("@resolve-js/core");

var _runtimeBase = require("@resolve-js/runtime-base");

var _getSecretsManager = require("../../runtime/get-secrets-manager");

var _getEventStore = require("../../runtime/get-event-store");

var _getSagaRuntime = require("../../runtime/get-saga-runtime");

var _mockSideEffects = require("../../runtime/mock-side-effects");

var _getReadModelAdapter = require("../../runtime/get-read-model-adapter");

var _assertions = require("../../utils/assertions");

var _utils = require("../../runtime/utils");

var promisedAssertion = function promisedAssertion(assertion, result, error) {
  return new Promise(function (resolve, reject) {
    return assertion(resolve, reject, result, error, false);
  });
};

var makeTestEnvironment = function makeTestEnvironment(context) {
  var mockedCommandImplementations = new Map();
  var mockedQueryImplementations = new Map();
  var executed = false;
  var useRealSideEffects = false;
  var sideEffectsStartTimestamp = 0;
  var assertions = [];
  var secretsManager = (0, _getSecretsManager.getSecretsManager)();
  var completeTest;
  var failTest;

  var setSecretsManager = function setSecretsManager(value) {
    secretsManager = value;
  };

  var allowSideEffects = function allowSideEffects() {
    useRealSideEffects = true;
  };

  var setSideEffectsStartTimestamp = function setSideEffectsStartTimestamp(value) {
    sideEffectsStartTimestamp = value;
  };

  var addAssertion = function addAssertion(value) {
    assertions.push((0, _lodash["default"])(promisedAssertion, value));
  };

  var isExecuted = function isExecuted() {
    return executed;
  };

  var mockCommandImplementation = function mockCommandImplementation(aggregateName, type, implementation) {
    mockedCommandImplementations.set((0, _utils.getCommandImplementationKey)({
      type: type,
      aggregateName: aggregateName
    }), implementation);
  };

  var mockQueryImplementation = function mockQueryImplementation(modelName, resolverName, implementation) {
    mockedQueryImplementations.set((0, _utils.getQueryImplementationKey)({
      modelName: modelName,
      resolverName: resolverName
    }), implementation);
  };

  var promise = new Promise(function (resolve, reject) {
    completeTest = resolve;
    failTest = reject;
  });

  var execute = /*#__PURE__*/function () {
    var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4() {
      var result, saga, events, adapter, encryption, actualEncryption, actualAdapter, isNext, domain, liveErrors, monitoring, errors, executor, eventstoreAdapter, runtime, status, liveErrorIndex, _error2, summaryError;

      return _regenerator["default"].wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              executed = true;
              result = {
                commands: [],
                scheduledCommands: [],
                queries: [],
                sideEffects: []
              };
              saga = context.saga, events = context.events, adapter = context.adapter, encryption = context.encryption;
              actualEncryption = encryption != null ? encryption : /*#__PURE__*/(0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
                return _regenerator["default"].wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        return _context.abrupt("return", null);

                      case 1:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              if (!(adapter != null)) {
                _context4.next = 8;
                break;
              }

              _context4.t0 = adapter;
              _context4.next = 11;
              break;

            case 8:
              _context4.next = 10;
              return (0, _getReadModelAdapter.getReadModelAdapter)();

            case 10:
              _context4.t0 = _context4.sent;

            case 11:
              actualAdapter = _context4.t0;
              isNext = false;
              domain = (0, _core.initDomain)({
                viewModels: [],
                readModels: [],
                aggregates: [],
                sagas: [{
                  name: saga.name,
                  handlers: saga.handlers,
                  sideEffects: useRealSideEffects ? saga.sideEffects : (0, _mockSideEffects.mockSideEffects)(result, saga.sideEffects),
                  connectorName: 'ADAPTER_NAME',
                  encryption: actualEncryption
                }],
                apiHandlers: []
              });
              liveErrors = [];
              monitoring = {
                group: function group(config) {
                  if (config.Part !== 'ReadModelProjection') {
                    return monitoring;
                  }

                  return (0, _extends2["default"])({}, monitoring, {
                    error: function (_error) {
                      function error(_x) {
                        return _error.apply(this, arguments);
                      }

                      error.toString = function () {
                        return _error.toString();
                      };

                      return error;
                    }(function (error) {
                      liveErrors.push(error);
                    }),
                    execution: function execution(error) {
                      if (error != null) {
                        liveErrors.push(error);
                      }
                    }
                  });
                },
                time: function time() {
                  return void 0;
                },
                timeEnd: function timeEnd() {
                  return void 0;
                },
                error: function error() {
                  return void 0;
                },
                execution: function execution() {
                  return void 0;
                },
                publish: function () {
                  var _publish = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2() {
                    return _regenerator["default"].wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            return _context2.abrupt("return", void 0);

                          case 1:
                          case "end":
                            return _context2.stop();
                        }
                      }
                    }, _callee2);
                  }));

                  function publish() {
                    return _publish.apply(this, arguments);
                  }

                  return publish;
                }(),
                duration: function duration() {
                  return void 0;
                },
                custom: function custom() {
                  return void 0;
                },
                rate: function rate() {
                  return void 0;
                },
                getMetrics: function getMetrics() {
                  return {
                    metrics: []
                  };
                },
                clearMetrics: function clearMetrics() {
                  return void 0;
                }
              };
              errors = [];
              executor = null;
              _context4.prev = 18;
              _context4.next = 21;
              return (0, _getEventStore.getEventStore)(events);

            case 21:
              eventstoreAdapter = _context4.sent;
              runtime = (0, _getSagaRuntime.getSagaRuntime)(result, {
                commands: mockedCommandImplementations,
                queries: mockedQueryImplementations
              }, domain.sagaDomain.schedulerName, secretsManager, monitoring, sideEffectsStartTimestamp);
              executor = (0, _runtimeBase.createQueryExecutor)({
                getEventSubscriberDestination: function getEventSubscriberDestination() {
                  return 'LOCAL';
                },
                applicationName: 'APP_NAME',
                readModelConnectors: {
                  ADAPTER_NAME: actualAdapter
                },
                getVacantTimeInMillis: function getVacantTimeInMillis() {
                  return 0x7fffffff;
                },
                invokeBuildAsync: function () {
                  var _invokeBuildAsync = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3() {
                    return _regenerator["default"].wrap(function _callee3$(_context3) {
                      while (1) {
                        switch (_context3.prev = _context3.next) {
                          case 0:
                            isNext = true;

                          case 1:
                          case "end":
                            return _context3.stop();
                        }
                      }
                    }, _callee3);
                  }));

                  function invokeBuildAsync() {
                    return _invokeBuildAsync.apply(this, arguments);
                  }

                  return invokeBuildAsync;
                }(),
                eventstoreAdapter: eventstoreAdapter,
                readModelsInterop: domain.sagaDomain.acquireSagasInterop(runtime),
                viewModelsInterop: {},
                performanceTracer: null,
                loadReadModelProcedure: function loadReadModelProcedure() {
                  return Promise.resolve(null);
                }
              });
              _context4.prev = 24;
              _context4.next = 27;
              return executor.subscribe({
                modelName: saga.name,
                subscriptionOptions: {
                  eventTypes: null,
                  aggregateIds: null
                }
              });

            case 27:
              _context4.next = 29;
              return executor.resume({
                modelName: saga.name
              });

            case 29:
              isNext = false;
              _context4.next = 32;
              return executor.build({
                modelName: saga.name
              });

            case 32:
              if (isNext) {
                _context4.next = 29;
                break;
              }

            case 33:
              _context4.next = 35;
              return executor.status({
                modelName: saga.name
              });

            case 35:
              status = _context4.sent;

              if (!(status.status === 'error')) {
                _context4.next = 47;
                break;
              }

              if (Array.isArray(status.errors)) {
                _context4.next = 39;
                break;
              }

              throw new Error('Unknown error');

            case 39:
              liveErrorIndex = status.errors.length === 1 ? liveErrors.findIndex(function (err) {
                return err.message === status.errors[0].message;
              }) : -1;

              if (!(liveErrorIndex < 0)) {
                _context4.next = 46;
                break;
              }

              _error2 = new Error(status.errors.map(function (err) {
                return err.message;
              }).join('\n'));
              _error2.stack = status.errors.map(function (err) {
                return err.stack;
              }).join('\n');
              throw _error2;

            case 46:
              throw liveErrors[liveErrorIndex];

            case 47:
              _context4.next = 52;
              break;

            case 49:
              _context4.prev = 49;
              _context4.t1 = _context4["catch"](24);
              errors.push(_context4.t1);

            case 52:
              _context4.prev = 52;
              _context4.next = 55;
              return executor.unsubscribe({
                modelName: saga.name
              });

            case 55:
              _context4.next = 60;
              break;

            case 57:
              _context4.prev = 57;
              _context4.t2 = _context4["catch"](52);
              errors.push(_context4.t2);

            case 60:
              _context4.next = 65;
              break;

            case 62:
              _context4.prev = 62;
              _context4.t3 = _context4["catch"](18);
              errors.push(_context4.t3);

            case 65:
              _context4.prev = 65;

              if (!(executor != null)) {
                _context4.next = 75;
                break;
              }

              _context4.prev = 67;
              _context4.next = 70;
              return executor.dispose();

            case 70:
              _context4.next = 75;
              break;

            case 72:
              _context4.prev = 72;
              _context4.t4 = _context4["catch"](67);
              errors.push(_context4.t4);

            case 75:
              return _context4.finish(65);

            case 76:
              if (!(errors.length === 0)) {
                _context4.next = 92;
                break;
              }

              if (!(assertions.length > 0)) {
                _context4.next = 89;
                break;
              }

              _context4.prev = 78;
              _context4.next = 81;
              return Promise.all(assertions.map(function (assertion) {
                return assertion(result, null);
              }));

            case 81:
              completeTest(result);
              _context4.next = 87;
              break;

            case 84:
              _context4.prev = 84;
              _context4.t5 = _context4["catch"](78);
              failTest(_context4.t5);

            case 87:
              _context4.next = 90;
              break;

            case 89:
              (0, _assertions.defaultAssertion)(completeTest, failTest, result, null);

            case 90:
              _context4.next = 108;
              break;

            case 92:
              summaryError = errors[0];

              if (errors.length > 1) {
                summaryError = new Error(errors.map(function (err) {
                  return err.message;
                }).join('\n'));
                summaryError.stack = errors.map(function (err) {
                  return err.stack;
                }).join('\n');
              } // eslint-disable-next-line no-console


              console.error(summaryError);

              if (!(assertions.length > 0)) {
                _context4.next = 107;
                break;
              }

              _context4.prev = 96;
              _context4.next = 99;
              return Promise.all(assertions.map(function (assertion) {
                return assertion(result, errors[0]);
              }));

            case 99:
              failTest(errors[0]);
              _context4.next = 105;
              break;

            case 102:
              _context4.prev = 102;
              _context4.t6 = _context4["catch"](96);
              failTest(_context4.t6);

            case 105:
              _context4.next = 108;
              break;

            case 107:
              (0, _assertions.defaultAssertion)(completeTest, failTest, result, errors[0]);

            case 108:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, null, [[18, 62, 65, 76], [24, 49], [52, 57], [67, 72], [78, 84], [96, 102]]);
    }));

    return function execute() {
      return _ref.apply(this, arguments);
    };
  }();

  setImmediate(execute);
  return {
    setSecretsManager: setSecretsManager,
    allowSideEffects: allowSideEffects,
    setSideEffectsStartTimestamp: setSideEffectsStartTimestamp,
    addAssertion: addAssertion,
    isExecuted: isExecuted,
    promise: promise,
    mockCommandImplementation: mockCommandImplementation,
    mockQueryImplementation: mockQueryImplementation
  };
};

exports.makeTestEnvironment = makeTestEnvironment;
//# sourceMappingURL=make-test-environment.js.map